NmTimerUpdater subclass: #NmWorldMapLocationBroadcaster
	instanceVariableNames: 'broadcastClient'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NetMorph-Server-MapServer'!

!NmWorldMapLocationBroadcaster methodsFor: 'accessing' stamp: 'mu 5/29/2005 03:00'!
broadcastClient
	broadcastClient ifNil: [broadcastClient _ RbtClient new].
	^broadcastClient! !


!NmWorldMapLocationBroadcaster methodsFor: 'private' stamp: 'mu 5/29/2005 02:48'!
broadcastingIntervalSeconds
	^(self intervalSeconds // self broadcastingPorts size) max: 1! !

!NmWorldMapLocationBroadcaster methodsFor: 'private' stamp: 'mu 6/3/2005 03:23'!
broadcastingPorts
	^NmServerSettings broadcastingWarpPorts! !

!NmWorldMapLocationBroadcaster methodsFor: 'private' stamp: 'mu 5/29/2005 03:52'!
locationMessage
	^NmBaseUtil localAddress, ':', NmServerSettings mapServerPort asString! !


!NmWorldMapLocationBroadcaster methodsFor: 'actions' stamp: 'mu 8/29/2005 11:31'!
update
	NmServerManager isMapServerRunning ifFalse: [^self].
	self showDebugUpdateMessage.
	self broadcastingPorts
		do: [:port | 
			self broadcastClient bindForBroadcast: port.
			self broadcastClient sendData: self locationMessage.
			(Delay forSeconds: self broadcastingIntervalSeconds) wait]! !


!NmWorldMapLocationBroadcaster methodsFor: 'testing' stamp: 'mu 8/22/2005 10:30'!
shouldRunAlways
	^true! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

NmWorldMapLocationBroadcaster class
	instanceVariableNames: ''!

!NmWorldMapLocationBroadcaster class methodsFor: 'constants' stamp: 'mu 5/30/2005 18:52'!
defaultIntervalSeconds
	^ 12! !


NmSoapServer subclass: #NmWorldMapServer
	instanceVariableNames: 'worldMaps clearProcess storeProcess'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NetMorph-Server-MapServer'!

!NmWorldMapServer methodsFor: 'accessing' stamp: 'mu 6/15/2005 23:54'!
backupFilename
	^NmBaseUtil shortImagePrefix, '.AllWorldMaps.bak'! !

!NmWorldMapServer methodsFor: 'accessing' stamp: 'mu 8/26/2005 14:34'!
clearingIntervalSeconds
	^NmServerSettings mapClearingIntervalSeconds! !

!NmWorldMapServer methodsFor: 'accessing' stamp: 'mu 6/15/2005 23:43'!
storingIntervalSeconds
	^self class defaultStoringIntervalSeconds! !

!NmWorldMapServer methodsFor: 'accessing' stamp: 'mu 1/21/2005 01:16'!
worldMaps
	worldMaps ifNil: [worldMaps _ Dictionary new].
	^worldMaps! !

!NmWorldMapServer methodsFor: 'accessing' stamp: 'mu 8/31/2005 13:04'!
worldMapsAt: mapName 
	^ self worldMaps
		at: mapName
		ifAbsentPut: [NmWorldMap new]! !


!NmWorldMapServer methodsFor: 'clearning' stamp: 'mu 5/4/2005 18:48'!
clearMapsNotUsed
	self clearMapsNotUsedFor: self clearingIntervalSeconds.
	! !

!NmWorldMapServer methodsFor: 'clearning' stamp: 'mu 6/15/2005 00:06'!
clearMapsNotUsedFor: intervalSeconds 
	| seconds emptyMaps emptyMapKeys |
	seconds _ intervalSeconds.
	self worldMaps
		do: [:each | each removeNotUpdatedFor: seconds].
	emptyMaps _ self worldMaps
				select: [:each | each isEmpty].
	emptyMaps
		keysAndValuesDo: [:name :map | map forgetNamed: name useServer: false].
	emptyMapKeys _ emptyMaps keys.
	emptyMapKeys
		do: [:each | self worldMaps
				removeKey: each
				ifAbsent: []]! !

!NmWorldMapServer methodsFor: 'clearning' stamp: 'mu 8/29/2005 09:56'!
clingTogether
	NmServerSettings autoClingTogether ifFalse: [^self].
	self worldMaps
		do: [:each | 
			each clingTogetherAloneEntries].! !


!NmWorldMapServer methodsFor: 'actions' stamp: 'mu 1/21/2005 02:03'!
getMapStringNamed: mapName 
	| map |
	map _ self worldMaps
				at: mapName
				ifAbsent: [].
	^ map isNil
		ifFalse: [map csvString]! !

!NmWorldMapServer methodsFor: 'actions' stamp: 'mu 5/7/2005 10:22'!
getVersionNamed: mapName 
	| map |
	map _ self worldMaps
				at: mapName
				ifAbsent: [].
	^ map isNil
		ifFalse: [map version]! !

!NmWorldMapServer methodsFor: 'actions' stamp: 'mu 5/4/2005 02:29'!
listAllMapNames
	self clearMapsNotUsed.
	^self worldMaps keys asSortedCollection asArray
	
	! !

!NmWorldMapServer methodsFor: 'actions' stamp: 'mu 9/1/2005 07:33'!
putMapString: mapCSVString named: mapName 
	| map tmpMap |
	map := self worldMapsAt: mapName.
	map isLocked ifTrue: [^#locked].
	tmpMap := NmWorldMap fromCsvString: mapCSVString.
	tmpMap map = map map ifTrue: [^#same]. 
	map fromCsvString: mapCSVString.
	map mapName: mapName.
	map storeNamed: mapName useServer: false.
	NmTranscript crShow: mapCSVString.
	^ #done! !

!NmWorldMapServer methodsFor: 'actions' stamp: 'mu 8/31/2005 12:32'!
removeEntryWith: uuidStr named: mapName
	| map |
	map := self worldMapsAt: mapName.
	map isLocked ifTrue: [^#locked].
	map
		removeKey: uuidStr
		ifAbsent: [^ #none].
	map storeNamed: mapName useServer: false.
	^ #done! !

!NmWorldMapServer methodsFor: 'actions' stamp: 'mu 8/26/2005 14:53'!
updateEntryWith: mapEntryString named: mapName 
	| worldMap originalEntry newEntry |
	self clearMapsNotUsed.
	worldMap := self worldMapsAt: mapName.
	worldMap isLocked ifTrue: [^worldMap csvString].
	newEntry := (NmWorldMapEntry fromCsvString: mapEntryString) unchanged.
	originalEntry := worldMap at: newEntry id.
	originalEntry = newEntry
		ifTrue: [originalEntry resetLastChangedTime]
		ifFalse: [worldMap registerDodgeIfConflict: newEntry.
			worldMap storeNamed: mapName useServer: false].
	^ worldMap csvString! !


!NmWorldMapServer methodsFor: 'initializing' stamp: 'mu 9/2/2005 16:45'!
initClearProcess
	| cleaner |
	clearProcess notNil
		ifTrue: [clearProcess release].
	cleaner := [:up | self clearMapsNotUsed.
			self clingTogether].
	clearProcess := NmBlockTimerUpdater
				named: self name , ':Clear'
				updateBlock: cleaner
				intervalSeconds: self clearingIntervalSeconds
				priority: Processor userSchedulingPriority - 2.
	clearProcess start! !

!NmWorldMapServer methodsFor: 'initializing' stamp: 'mu 9/1/2005 11:35'!
initialize
	"self default initialize"
	super initialize.
	self release.
	worldMaps := nil.
	self initProcesses! !

!NmWorldMapServer methodsFor: 'initializing' stamp: 'mu 9/1/2005 11:35'!
initProcesses
	self initClearProcess.
	self initStoreProcess! !

!NmWorldMapServer methodsFor: 'initializing' stamp: 'mu 9/2/2005 16:46'!
initStoreProcess
	| saver |
	storeProcess notNil
		ifTrue: [storeProcess release].
	saver := [:up | self store].
	storeProcess := NmBlockTimerUpdater
				named: self name , ':Store'
				updateBlock: saver
				intervalSeconds: self storingIntervalSeconds
				priority: Processor userSchedulingPriority - 5.
	storeProcess start! !

!NmWorldMapServer methodsFor: 'initializing' stamp: 'mu 9/2/2005 08:05'!
release
	clearProcess ifNotNilDo: [:p | p release].
	clearProcess := nil.
	storeProcess ifNotNilDo: [:p | p release].
	storeProcess := nil
	! !


!NmWorldMapServer methodsFor: 'storing/loading' stamp: 'mu 6/15/2005 23:56'!
load
	| loaded |
	loaded _ NmBaseUtil loadFrom: FileDirectory default named: self backupFilename.
	loaded notNil
		ifTrue: [worldMaps _ loaded.
			NmTranscript crShow: '## All MapServer data were loaded: ' , self backupFilename]! !

!NmWorldMapServer methodsFor: 'storing/loading' stamp: 'mu 6/16/2005 18:58'!
store
	self worldMaps isEmpty ifTrue: [^self].
	NmBaseUtil store: self worldMaps to: FileDirectory default named: self backupFilename! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

NmWorldMapServer class
	instanceVariableNames: ''!

!NmWorldMapServer class methodsFor: 'constants' stamp: 'mu 6/16/2005 00:57'!
defaultStoringIntervalSeconds
	"1 hour"
	^60 * 60
	! !


!NmWorldMapServer class methodsFor: 'utilities' stamp: 'mu 4/10/2005 02:08'!
exploreMaps
	"NmWorldMapServer exploreMaps"
	self default worldMaps explore! !

!NmWorldMapServer class methodsFor: 'utilities' stamp: 'mu 6/15/2005 23:39'!
load
	"NmWorldMapServer load"
	^self default load! !

!NmWorldMapServer class methodsFor: 'utilities' stamp: 'mu 2/2/2005 00:51'!
release
	"self release"
	self default release! !

!NmWorldMapServer class methodsFor: 'utilities' stamp: 'mu 6/15/2005 23:56'!
store
	"NmWorldMapServer store"
	^self default store! !


!NmWorldMapServer class methodsFor: 'class initialization' stamp: 'mu 3/12/2003 00:19'!
initialize
	self basicInitialize! !

!NmWorldMapServer class methodsFor: 'class initialization' stamp: 'mu 2/2/2005 00:52'!
obsolete
	self release.
	super obsolete! !


!NmWorldMapServer class methodsFor: 'registering' stamp: 'mu 5/7/2005 10:23'!
registerServices
	"NmWorldMapServer registerServices"
	self registerServiceNamed: #'putMapString:named:' paramNames: #(#mapCSVString #mapName ).
	self registerServiceNamed: #'getMapStringNamed:' paramNames: #(#mapName ).
	self registerServiceNamed: #'getVersionNamed:' paramNames: #(#mapName ).
	self registerServiceNamed: #listAllMapNames paramNames: #().
	self registerServiceNamed: #'updateEntryWith:named:' paramNames: #(#mapEntryString #mapName ).
	self registerServiceNamed: #'removeEntryWith:named:' paramNames: #(#uuidStr #mapName )! !

NmWorldMapServer initialize!